#include "../include/mp3infpExtr.h"
#include "TfxExtr.h"

#include <tchar.h>

TEString	_GFormatW(LPCWSTR lpcFormat, va_list args);

TEString	TEConvertToTEString(const char *p)
{
	if (p == NULL) {
		return	TEString();
	}
	const	size_t	len	= mbstowcs((wchar_t *)NULL, p, 0);
	if (len == (size_t)-1) {
		return	TEString();
	}
	wchar_t	*s	= new wchar_t[len+1];
	mbstowcs(s, p, len+1);
	TEString	str(s);
	delete[] s;
	return	str;
}

bool	TfxIsHeadMatchCase(const TEString &strTarget, LPCTSTR lpcMatch, size_t len) {
	return	_tcsncmp(strTarget.c_str(), lpcMatch, len) == 0;
}

bool	TfxIsHeadMatchCase(const TEString &strTarget, const TEString &strMatch) {
	return	TfxIsHeadMatchCase(strTarget, strMatch.c_str(), strMatch.size());
}

static	const	TEString	strUNC		= _T("\\\\");
static	const	TEString	strPrefix	= _T("\\\\?\\");

TEString TETouchLongPathName(const TEString & strPath)
{
	if (!TfxIsHeadMatchCase(strPath, strUNC) && strPath.length() >= 2 && strPath[1] == ':') {
		return	strPrefix + strPath;
	}
	return	strPath;
}

TEString	TEFormatW(LPCWSTR lpcFormat, ...)
{
	va_list	args;
	va_start(args, lpcFormat);
	return	_GFormatW(lpcFormat, args);
}

TEString	_GFormatW(LPCWSTR lpcFormat, va_list args)
{
	int		nBuf	= 128;
	wchar_t	*pBuf	= NULL;
	TEString	str;
	try {
		while (true) {
			pBuf	= new wchar_t [nBuf];
			if (pBuf == NULL) {
				str	= L"TfxExtr.cpp: Memory allocation failure.";
				break;
			}
			int	nForm	= _vsnwprintf(pBuf, nBuf, lpcFormat, args);
			if (nForm >= nBuf || nForm < 0) {
				delete	[]	pBuf;
				pBuf	= NULL;
				if (nBuf >= INT_MAX/4) {
					str	= L"TfxExtr.cpp: Memory size too big for: " + TEString(lpcFormat);
					break;
				}
				nBuf	*= 2;
			}
			else {
				str		= pBuf;
				delete	[]	pBuf;
				pBuf	= NULL;
				break;
			}
		}
	}
	catch (std::bad_alloc) {
	}
	if (pBuf) {
		delete	[]	pBuf;
	}
	return	str;
}

FILE*	TEFOpen(LPCTSTR lpcFile, LPCTSTR lpcAttr)
{
	const TEString	strFL	= lpcFile;
	const TEString	strLF	= TETouchLongPathName(strFL);
	return	_tfopen(strLF.c_str(), lpcAttr);
}

HANDLE	TECreateFile(LPCWSTR lpFileName, DWORD dwDesiredAccess, DWORD dwShareMode, LPSECURITY_ATTRIBUTES lpSecurityAttributes, DWORD dwCreationDisposition, DWORD dwFlagsAndAttributes, HANDLE hTemplateFile)
{
	const TEString	strFL	= lpFileName;
	const TEString	strLF	= TETouchLongPathName(strFL);
	return	CreateFile(strLF.c_str(), dwDesiredAccess, dwShareMode, lpSecurityAttributes, dwCreationDisposition, dwFlagsAndAttributes, hTemplateFile);
}

BOOL	TEMoveFile(LPCTSTR lpExistingFileName, LPCTSTR lpNewFileName)
{
	const TEString	strFL1	= lpExistingFileName;
	const TEString	strLF1	= TETouchLongPathName(strFL1);
	const TEString	strFL2	= lpNewFileName;
	const TEString	strLF2	= TETouchLongPathName(strFL2);
	return	MoveFile(strLF1.c_str(), strLF2.c_str());
}

BOOL	TEDeleteFile(LPCTSTR lpFileName)
{
	const TEString	strFL	= lpFileName;
	const TEString	strLF	= TETouchLongPathName(strFL);
	return	DeleteFile(strLF.c_str());
}
